@@ -1242,7 +1242,7 @@ gnome_vfs_format_uri_for_display_internal (const char *uri,
  * 	file: uri with fragments should appear as file:uri.
  * 	All other uri appear as expected.
  *
- * Return value: a string which represents @uri and can be displayed.
+ * Return value: a newly allocated string which represents @uri and can be displayed.
  *
  * Since: 2.2
  */
@@ -1258,28 +1258,25 @@ gnome_vfs_format_uri_for_display (const char *uri)
 	return gnome_vfs_format_uri_for_display_internal (uri, utf8, charset);
 }
 
-static gboolean
-is_valid_scheme_character (char c)
-{
-	return g_ascii_isalnum (c) || c == '+' || c == '-' || c == '.';
-}
-
 static gboolean
 has_valid_scheme (const char *uri)
 {
+	char *scheme;
 	const char *p;
+	gboolean ret;
 
-	p = uri;
+	ret = FALSE;
 
-	if (!is_valid_scheme_character (*p)) {
-		return FALSE;
+	p = strchr (uri, ':');
+	if (p != NULL) {
+		scheme = g_strndup (uri, p - uri);
+		if (gnome_vfs_method_get (scheme) != NULL) {
+			ret = TRUE;
+		}
+		g_free (scheme);
 	}
 
-	do {
-		p++;
-	} while (is_valid_scheme_character (*p));
-
-	return *p == ':';
+	return ret;
 }
 
 static char *
